import * as searchSettingServices from '@12333/services/api/SearchSetting';
import { useQuery, useQueryClient } from '@tanstack/vue-query';
import { SearchType } from '@12333/constants';
import { MaybeRef, computed } from 'vue';

type UseSearchSettingTypeOptions = {
  searchType: number;
  belongType?: number;
  onSuccess?: (data: API.GetTypeSearchSettingList[]) => any;
  enabled?: MaybeRef<boolean>;
};

export function useSearchSettingType({
  searchType,
  belongType = null,
  onSuccess,
  enabled = true,
}: UseSearchSettingTypeOptions) {
  const { data, refetch } = useQuery({
    queryKey: ['searchSettingServices/getTypeSearchSettingList', { searchType, belongType }],
    queryFn: async () => {
      return await searchSettingServices.getTypeSearchSettingList(
        {
          searchType: searchType,
          belongType: belongType,
        },
        { showLoading: false }
      );
    },
    placeholderData: () => [] as API.GetTypeSearchSettingList[],
    onSuccess(data) {
      onSuccess?.(data);
    },
    enabled,
  });

  const queryClient = useQueryClient();

  async function ensureSearchSettingType() {
    return await queryClient.ensureQueryData({
      queryKey: [
        'searchSettingServices/getTypeSearchSettingList',
        { searchType: searchType, belongType: belongType },
      ],
    });
  }

  function getSearchSettingTypeNameById(id: string) {
    return data.value.find((x) => x.id === id)?.name ?? '';
  }

  return {
    searchSettingTypeList: data,
    ensureSearchSettingType,
    refetchSearchSettingType: refetch,
    getSearchSettingTypeNameById,
  };
}

type UseAllSearchSettingList = API.GetSearchSettingListInput & {
  enabled?: MaybeRef<boolean>;
};

export function useAllSearchSettingList(options: UseAllSearchSettingList) {
  const { enabled = true, ...params } = options;

  const { data: allSearchSettingList } = useQuery({
    queryKey: ['searchSettingServices/getAllSearchSettingList', params],
    queryFn: async () => {
      return await searchSettingServices.getAllSearchSettingList(params, { showLoading: false });
    },
    initialData: () => [] as API.GetSearchSettingList[],
    enabled,
  });

  return {
    allSearchSettingList,
  };
}

type UseWorkOfTypeOptions = {
  enabled?: MaybeRef<boolean>;
};

export function useWorkOfType(options: UseWorkOfTypeOptions = {}) {
  const { enabled = true } = options;

  const { allSearchSettingList: industryCategoryList } = useAllSearchSettingList({
    searchType: SearchType.IndustryCategory,
    enabled,
    status: true,
  });

  const { allSearchSettingList: flattenWorkOfTypeList } = useAllSearchSettingList({
    searchType: SearchType.Work,
    enabled,
    status: true,
  });

  const workOfTypeList = computed<API.GetSearchSettingList[]>(() => {
    if (industryCategoryList.value?.length > 0) {
      let list = [];
      industryCategoryList.value.forEach((x) => {
        list.push({
          ...x,
          children: flattenWorkOfTypeList.value.filter((w) => w.parentId === x.id),
        });
      });
      return list;
    }
    return [];
  });

  const workOfTypeRecommendList = computed<API.GetSearchSettingList[]>(() => {
    if (flattenWorkOfTypeList.value?.length > 0) {
      return flattenWorkOfTypeList.value.filter((w) => w.isRecommend);
    }
    return [];
  });

  function getWorkOfTypeNameById(id: string) {
    return flattenWorkOfTypeList.value.find((x) => x.id === id)?.name ?? '';
  }

  const MaxSelectWorkOfTypeLimit = 6;

  return {
    workOfTypeList: workOfTypeList,
    flattenWorkOfTypeList,
    workOfTypeRecommendList,
    getWorkOfTypeNameById,
    MaxSelectWorkOfTypeLimit,
  };
}
